@@ -1,17 +1,31 @@
 /* SleepLib Common Functions
  *
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2025 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
  * for more details. */
 
+#define TEST_MACROS_ENABLEDoff
+#include <test_macros.h>
+
 #include <QDateTime>
+#include <QTimeZone>
 #include <QDir>
 #include <QThread>
 #ifndef BROKEN_OPENGL_BUILD
-#include <QGLWidget>
-#endif
+    #if QT_VERSION < QT_VERSION_CHECK(5,4,0)
+        #include <QGLWidget>
+    #elif  QT_VERSION < QT_VERSION_CHECK(6,0,0)
+        #include <QOpenGLWidget>
+    #else
+        #include <QtOpenGLWidgets/QOpenGLWidget>
+        #include <QOpenGLFunctions>
+        #include <QSurfaceFormat>
+        #include <QOffscreenSurface>
+    #endif
+#endif //BROKEN_OPENGL_BUILD
 #include <QOpenGLFunctions>
 #include <QDebug>
 #include <QDir>
@@ -124,23 +138,30 @@ void setCurrentGFXEngine(GFXEngine e)
 QString getOpenGLVersionString()
 {
     static QString glversion;
-   if (glversion.isEmpty()) {
+    if (glversion.isEmpty()) {
 
-#ifdef BROKEN_OPENGL_BUILD
+    #ifdef BROKEN_OPENGL_BUILD
         glversion="LegacyGFX";
         qDebug() << "This LegacyGFX build has been created without the need for OpenGL";
-#else
-
-        QGLWidget w;
-        w.makeCurrent();
-
-        QOpenGLFunctions f;
-        f.initializeOpenGLFunctions();
-        glversion = QString(QLatin1String(reinterpret_cast<const char*>(f.glGetString(GL_VERSION))));
-//        qDebug() << "Graphics Engine:" << glversion;
-#endif
-   }
-   return glversion;
+    #else
+        #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+            QGLWidget w;
+            w.makeCurrent();
+            QOpenGLFunctions f;
+            f.initializeOpenGLFunctions();
+            glversion = QString(QLatin1String(reinterpret_cast<const char*>(f.glGetString(GL_VERSION))));
+        #else
+            QOffscreenSurface surf;
+            surf.create();
+            QOpenGLContext ctx;
+            ctx.create();
+            ctx.makeCurrent(&surf);
+            glversion = (const char*)ctx.functions()->glGetString(GL_VERSION);
+            surf.destroy();
+        #endif
+    #endif
+    }
+    return glversion;
 }
 
 float getOpenGLVersion()
@@ -184,15 +205,52 @@ QString getGraphicsEngine()
     return gfxEngine;
 }
 
+QString getCompilerVersion()
+{
+    #ifdef __clang_version__
+        return QString("clang++:%1").arg(__clang_version__);
+    #elif defined(__MINGW64__)
+        return QString("MINGW64:%1").arg(__VERSION__);
+    #elif defined(__MINGW32__)
+        return QString("MINGW32:%1").arg(__VERSION__);
+    #elif defined (__GNUG__) or defined (__GNUC__)
+        return QString("GNU C++:%1").arg(__VERSION__);
+    #else
+        return QString();
+    #endif
+}
+
+QString getCppVersion() {
+    #if __cplusplus > 202002L
+        return  "Using C++23 or later";
+    #elif __cplusplus == 202002L
+        return  "Using C++20";
+    #elif __cplusplus >= 201703L
+        return "Using C++17";
+    #elif __cplusplus >= 201402L
+        return "Using C++14";
+    #elif __cplusplus >= 201103L
+        return "Using C++11";
+    #else
+        return QString();
+    #endif
+}
+
 QStringList buildInfo;
 
 QStringList makeBuildInfo (QString forcedEngine){
     // application name and version has already been added
     buildInfo << (QObject::tr("Built with Qt %1 on %2").arg(QT_VERSION_STR).arg(getBuildDateTime()));
+    QString compiler = getCompilerVersion();
+    if (compiler.length() >0 )
+        buildInfo << (QObject::tr("Compiler:") + " " + compiler);
+    if (!getCppVersion().isEmpty())
+        buildInfo << getCppVersion();
     buildInfo << QString("");
     buildInfo << (QObject::tr("Operating system:") + " " + QSysInfo::prettyProductName());
     buildInfo << (QObject::tr("Graphics Engine:") + " " + getOpenGLVersionString());
     buildInfo << (QObject::tr("Graphics Engine type:") + " " + getGraphicsEngine());
+
     if (forcedEngine != "")
         buildInfo << forcedEngine;
 
@@ -225,18 +283,18 @@ QString appResourcePath()
 //    QStringList paths = QStandardPaths::standardLocations(QStandardPaths::AppDataLocation);
     // Check the Appiication Path first, so we can execute out of the build directory
     QStringList paths;
-    // This one will be used if the Html and Translations folders 
+    // This one will be used if the Html and Translations folders
     // are in the same folder as  the OSCAR executable
     paths.append( QCoreApplication::applicationDirPath() );
-#ifdef Q_OS_LINUX    
+#ifdef Q_OS_LINUX 
     QString appName = QCoreApplication::applicationName();
     if (appName != QString("OSCAR"))
         appName = QString("OSCAR-test");
     paths.append( QString( "/usr/share/" ) + appName );
     paths.append( QString( "/usr/local/share/" ) + appName );
-#endif    
+#endif
     for (auto p = begin(paths); p != end(paths); ++p ) {
-        QString fname = *p+QString("/Translations/oscar_qt_fr.qm");
+        QString fname = *p+QString("/Html/about.html"); // was "/Translations/oscar_qt_fr.qm"  - Crimson Nape
         qDebug() << "Trying" << fname;
         QFileInfo f = QFileInfo(fname);
         if ( f.exists() ) {
@@ -247,7 +305,7 @@ QString appResourcePath()
     if ( path.size() == 0 )
         path = QCoreApplication::applicationDirPath();
 #endif
-        
+     
     return path;
 }
 
@@ -267,7 +325,7 @@ qint64 timezoneOffset()
 
     QDateTime d1 = QDateTime::currentDateTime();
     QDateTime d2 = d1;
-    d1.setTimeSpec(Qt::UTC);
+    d1.setTimeZone(QTimeZone("UTC"));
     _TZ_offset = d2.secsTo(d1);
     _TZ_offset *= 1000L;
     return _TZ_offset;
@@ -282,7 +340,7 @@ QString weightString(float kg, UnitSystem us)
     if (us == US_Metric) {
         return QString("%1kg").arg(kg, 0, 'f', 2);
     } else if (us == US_English) {
-        int oz = (kg * 1000.0) / (float)ounce_convert;
+        int oz = (kg * 1000.0) * (float)gram_ounce_convert;
         int lb = oz / 16.0;
         oz = oz % 16;
         return QString("%1lb %2oz").arg(lb, 0, 10).arg(oz);
@@ -329,9 +387,15 @@ void validateFont (QString which, int size, bool bold, bool italic) {
     // Get list of installed font families, including system font
     // Do this just once so we don't have to call font functions repeatedly
     // (This list includes private fonts)
+    #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QFontDatabase fontdatabase;
+    #endif
     if (installedFontFamilies.isEmpty()) {
-        installedFontFamilies = fontdatabase.families();
+        #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+            installedFontFamilies = fontdatabase.families();
+        #else
+            installedFontFamilies = QFontDatabase::families();
+        #endif
         }
     qDebug() << "validateFont found" << installedFontFamilies.count() << "installed font families";
 
@@ -362,7 +426,11 @@ void validateFont (QString which, int size, bool bold, bool italic) {
 
 #ifdef Q_OS_MAC
     // Don't allow private font to be set for anything other than Application font (Mac restricts use to UI)
+    #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     if (which != "Application" && fontdatabase.isPrivateFamily(desiredFont)) {
+    #else
+    if (which != "Application" && QFontDatabase::isPrivateFamily(desiredFont)) {
+    #endif
         desiredFont = "Helvetica";              // We assume "Helvetica" is universally available on Mac
         forceFont = true;
     }
@@ -379,7 +447,15 @@ void validateFont (QString which, int size, bool bold, bool italic) {
 }
 
 void setApplicationFont () {
+    if (mainwin == nullptr) {
+	qWarning() << "Warning: mainwin not set in setApplicationFont";
+	return;	
+    }
     qDebug() << "Application font starts out as" << QApplication::font();
+    if (p_pref == nullptr) {
+	qWarning() << "Warning: p_pref not set in setApplicationFont";
+	return;
+    }
     QFont font = QFont(((*p_pref)["Fonts_Application_Name"]).toString());
     font.setPointSize(((*p_pref)["Fonts_Application_Size"]).toInt());
     font.setWeight(((*p_pref)["Fonts_Application_Bold"]).toBool() ? QFont::Bold : QFont::Normal);
@@ -463,6 +539,7 @@ QString STR_UNIT_CMH2O;
 QString STR_UNIT_Hours;
 QString STR_UNIT_Minutes;
 QString STR_UNIT_Seconds;
+QString STR_UNIT_milliSeconds;
 QString STR_UNIT_h;
 QString STR_UNIT_m;
 QString STR_UNIT_s;
@@ -520,11 +597,14 @@ QString STR_TR_Orientation;
 QString STR_TR_Motion;
 
 
-// Machine type names.
+// Device type names.
 QString STR_TR_CPAP;    // Constant Positive Airway Pressure
 QString STR_TR_BIPAP;   // Bi-Level Positive Airway Pressure
 QString STR_TR_BiLevel; // Another name for BiPAP
 QString STR_TR_EPAP;    // Expiratory Positive Airway Pressure
+QString STR_TR_EEPAP;    // End Expiratory Positive Airway Pressure
+QString STR_TR_EEPAPLo;  // End-Expiratory Positive Airway Pressure, Low
+QString STR_TR_EEPAPHi;  // End-Expiratory Positive Airway Pressure, High
 QString STR_TR_EPAPLo;  // Expiratory Positive Airway Pressure, Low
 QString STR_TR_EPAPHi;  // Expiratory Positive Airway Pressure, High
 QString STR_TR_IPAP;    // Inspiratory Positive Airway Pressure
@@ -547,13 +627,13 @@ QString STR_TR_SA;      // Short form of SensAwake
 QString STR_TR_LE;      // Short form of Leak Event
 QString STR_TR_EP;      // Short form of Expiratory Puff
 QString STR_TR_VS;      // Short form of Vibratory Snore
-QString STR_TR_VS2;     // Short form of Secondary Vibratory Snore (Some Philips Respironics Machines have two sources)
+QString STR_TR_VS2;     // Short form of Secondary Vibratory Snore (Some Philips Respironics devices have two sources)
 QString STR_TR_RERA;    // Acronym for Respiratory Effort Related Arousal
 QString STR_TR_PP;      // Short form for Pressure Pulse
 QString STR_TR_P;       // Short form for Pressure Event
 QString STR_TR_RE;      // Short form of Respiratory Effort Related Arousal
 QString STR_TR_NR;      // Short form of Non Responding event? (forgot sorry)
-QString STR_TR_NRI;     // Sorry I Forgot.. it's a flag on Intellipap machines
+QString STR_TR_NRI;     // Sorry I Forgot.. it's a flag on Intellipap devices
 QString STR_TR_O2;      // SpO2 Desaturation
 QString STR_TR_PC;      // Short form for Pulse Change
 QString STR_TR_UF1;     // Short form for User Flag 1
@@ -663,28 +743,29 @@ void initializeStrings()
 
     STR_UNIT_M = QObject::tr(" m");
     STR_UNIT_CM = QObject::tr(" cm");
-    STR_UNIT_INCH = QObject::tr("\"");
+    STR_UNIT_INCH = QObject::tr("in");
     STR_UNIT_FOOT = QObject::tr("ft");
     STR_UNIT_POUND = QObject::tr("lb");
     STR_UNIT_OUNCE = QObject::tr("oz");
-    STR_UNIT_KG = QObject::tr("Kg");
+    STR_UNIT_KG = QObject::tr("kg");
     STR_UNIT_CMH2O = QObject::tr("cmH2O");
     STR_UNIT_Hours = QObject::tr("Hours");
     STR_UNIT_Minutes = QObject::tr("Minutes");
     STR_UNIT_Seconds = QObject::tr("Seconds");
+    STR_UNIT_milliSeconds = QObject::tr("milliSeconds");
     STR_UNIT_h = QObject::tr("h"); // hours shortform
     STR_UNIT_m = QObject::tr("m"); // minutes shortform
     STR_UNIT_s = QObject::tr("s"); // seconds shortform
     STR_UNIT_ms = QObject::tr("ms"); // milliseconds
     STR_UNIT_EventsPerHour = QObject::tr("Events/hr"); // Events per hour
-    STR_UNIT_Percentage = QObject::tr("%");
+    STR_UNIT_Percentage = QString("%");
     STR_UNIT_Hz = QObject::tr("Hz");          // Hertz
     STR_UNIT_BPM = QObject::tr("bpm");        // Beats per Minute
-    STR_UNIT_LPM = QObject::tr("L/min");      // Litres per Minute
+    STR_UNIT_LPM = QObject::tr("l/min");      // Litres per Minute
     STR_UNIT_Litres = QObject::tr("Litres");
     STR_UNIT_ml = QObject::tr("ml");        // millilitres
     STR_UNIT_BreathsPerMinute = QObject::tr("Breaths/min"); // Breaths per minute
-    STR_UNIT_Unknown = QObject::tr("?");
+    STR_UNIT_Unknown = QString("?");
     STR_UNIT_Ratio = QObject::tr("ratio");
     STR_UNIT_Severity = QObject::tr("Severity (0-1)");
     STR_UNIT_Degrees = QObject::tr("Degrees");
@@ -697,24 +778,24 @@ void initializeStrings()
     STR_MessageBox_PleaseNote = QObject::tr("Please Note");
 
     STR_Empty_NoData = QObject::tr("No Data Available");
-    STR_Empty_Brick = QObject::tr("Compliance Only :(");
+    STR_Empty_Brick = QObject::tr("Only Settings and Compliance Data Available");
     STR_Empty_NoGraphs = QObject::tr("Graphs Switched Off");
-    STR_Empty_SummaryOnly = QObject::tr("Summary Only :(");
+    STR_Empty_SummaryOnly = QObject::tr("Summary Data Only");
     STR_Empty_NoSessions = QObject::tr("Sessions Switched Off");
 
 
     // Dialog box options
-    STR_MessageBox_Yes = QObject::tr("&Yes");
-    STR_MessageBox_No = QObject::tr("&No");
-    STR_MessageBox_Cancel = QObject::tr("&Cancel");
-    STR_MessageBox_Destroy = QObject::tr("&Destroy");;
-    STR_MessageBox_Save = QObject::tr("&Save");
+    STR_MessageBox_Yes = QObject::tr("Yes");
+    STR_MessageBox_No = QObject::tr("No");
+    STR_MessageBox_Cancel = QObject::tr("Cancel");
+    STR_MessageBox_Destroy = QObject::tr("Destroy");;
+    STR_MessageBox_Save = QObject::tr("Save");
 
     STR_TR_BMI = QObject::tr("BMI");              // Short form of Body Mass Index
     STR_TR_Weight = QObject::tr("Weight");
-    STR_TR_Zombie = QObject::tr("Zombie");
+    STR_TR_Zombie = QObject::tr("Feeling");
     STR_TR_PulseRate = QObject::tr("Pulse Rate"); // Pulse / Heart rate
-    STR_TR_SpO2 = QObject::tr("SpO2");
+    STR_TR_SpO2 = QString("SpO2");
     STR_TR_Plethy = QObject::tr("Plethy");        // Plethysomogram
     STR_TR_Pressure = QObject::tr("Pressure");
 
@@ -729,11 +810,14 @@ void initializeStrings()
 
     STR_TR_Default = QObject::tr("Default");
 
-    // Machine type names.
+    // Device type names.
     STR_TR_CPAP = QObject::tr("CPAP");    // Constant Positive Airway Pressure
     STR_TR_BIPAP = QObject::tr("BiPAP");  // Bi-Level Positive Airway Pressure
     STR_TR_BiLevel = QObject::tr("Bi-Level"); // Another name for BiPAP
     STR_TR_EPAP = QObject::tr("EPAP");    // Expiratory Positive Airway Pressure
+    STR_TR_EEPAP = QObject::tr("EEPAP");    // End-Expiratory Positive Airway Pressure
+    STR_TR_EEPAPLo = QObject::tr("Min EEPAP");    // Lower End-Expiratory Positive Airway Pressure
+    STR_TR_EEPAPHi = QObject::tr("Max EEPAP");    // Higher End-Expiratory Positive Airway Pressure
     STR_TR_EPAPLo = QObject::tr("Min EPAP"); // Lower Expiratory Positive Airway Pressure
     STR_TR_EPAPHi = QObject::tr("Max EPAP"); // Higher Expiratory Positive Airway Pressure
     STR_TR_IPAP = QObject::tr("IPAP");    // Inspiratory Positive Airway Pressure
@@ -757,13 +841,13 @@ void initializeStrings()
     STR_TR_EP = QObject::tr("EP");    // Short form of Expiratory Puff
     STR_TR_VS = QObject::tr("VS");    // Short form of Vibratory Snore
     STR_TR_VS2 =
-        QObject::tr("VS2");  // Short form of Secondary Vibratory Snore (Some Philips Respironics Machines have two sources)
+        QObject::tr("VS2");  // Short form of Secondary Vibratory Snore (Some Philips Respironics devices have two sources)
     STR_TR_RERA = QObject::tr("RERA"); // Acronym for Respiratory Effort Related Arousal
     STR_TR_PP = QObject::tr("PP");    // Short form for Pressure Pulse
     STR_TR_P = QObject::tr("P");      // Short form for Pressure Event
     STR_TR_RE = QObject::tr("RE");    // Short form of Respiratory Effort Related Arousal
     STR_TR_NR = QObject::tr("NR");    // Short form of Non Responding event? (forgot sorry)
-    STR_TR_NRI = QObject::tr("NRI");  // Sorry I Forgot.. it's a flag on Intellipap machines
+    STR_TR_NRI = QObject::tr("NRI");  // Sorry I Forgot.. it's a flag on Intellipap devices
     STR_TR_O2 = QObject::tr("O2");    // SpO2 Desaturation
     STR_TR_PC = QObject::tr("PC");    // Short form for Pulse Change
     STR_TR_UF1 = QObject::tr("UF1");    // Short form for User Flag 1
@@ -814,7 +898,7 @@ void initializeStrings()
     STR_TR_PrRelief = QObject::tr("Pr. Relief"); // Pressure Relief
 
     STR_TR_Bookmarks = QObject::tr("Bookmarks");
-    STR_TR_OSCAR = QObject::tr("OSCAR");
+    STR_TR_OSCAR = QString("OSCAR");
     //STR_TR_AppVersion = QObject::tr("v%1").arg(getVersion());
 
     STR_TR_Mode = QObject::tr("Mode");
@@ -822,7 +906,7 @@ void initializeStrings()
     STR_TR_Brand = QObject::tr("Brand");
     STR_TR_Serial = QObject::tr("Serial");
     STR_TR_Series = QObject::tr("Series");
-    STR_TR_Machine = QObject::tr("Machine");
+    STR_TR_Machine = QObject::tr("Device");
     STR_TR_Channel = QObject::tr("Channel");
     STR_TR_Settings = QObject::tr("Settings");
 
