@@ -1,12 +1,16 @@
 /* OSCAR Main
  *
- * Copyright (c) 2019-2020 The OSCAR Team
- * Copyright (c) 2011-2018 Mark Watkins <mark@jedimark.net>
+ * Copyright (c) 2019-2025 The OSCAR Team
+ * Copyright (c) 2011-2018 Mark Watkins
  *
  * This file is subject to the terms and conditions of the GNU General Public
  * License. See the file COPYING in the main directory of the source code
  * for more details. */
 
+#define TEST_MACROS_ENABLEDoff
+#include <test_macros.h>
+#include <QtGlobal>
+
 #ifdef UNITTEST_MODE
 #include "tests/AutoTest.h"
 #endif
@@ -29,6 +33,9 @@
 #include "translation.h"
 #include "SleepLib/common.h"
 #include "SleepLib/deviceconnection.h"
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+#include "highresolution.h"
+#endif
 
 #include <ctime>
 #include <chrono>
@@ -46,8 +53,13 @@
 #include "SleepLib/loader_plugins/sleepstyle_loader.h"
 #include "SleepLib/loader_plugins/weinmann_loader.h"
 #include "SleepLib/loader_plugins/viatom_loader.h"
+#include "SleepLib/loader_plugins/prisma_loader.h"
+#include "SleepLib/loader_plugins/resvent_loader.h"
+#include "SleepLib/loader_plugins/vrem_loader.h"
+#include "SleepLib/loader_plugins/bmc_loader.h"
 
 MainWindow *mainwin = nullptr;
+extern bool openOk;
 
 int numFilesCopied = 0;
 
@@ -117,8 +129,8 @@ bool processPreferenceFile( QString path ) {
     QFile fl(fullpath);
     QFile tmp(fullpath+".tmp");
     QString line;
-    fl.open(QIODevice::ReadOnly);
-    tmp.open(QIODevice::WriteOnly);
+    openOk = fl.open(QIODevice::ReadOnly);
+    openOk = tmp.open(QIODevice::WriteOnly);
     QTextStream instr(&fl);
     QTextStream outstr(&tmp);
     bool isSleepyHead = false;
@@ -145,8 +157,8 @@ bool processFile( QString fullpath ) {
     QFile fl(fullpath);
     QFile tmp(fullpath+".tmp");
     QString line;
-    fl.open(QIODevice::ReadOnly);
-    tmp.open(QIODevice::WriteOnly);
+    openOk = fl.open(QIODevice::ReadOnly);
+    openOk = tmp.open(QIODevice::WriteOnly);
     QTextStream instr(&fl);
     QTextStream outstr(&tmp);
     while (instr.readLineInto(&line)) {
@@ -221,8 +233,8 @@ bool migrateFromSH(QString destDir) {
     qDebug() << "Number of files to migrate: " << numFiles;
 
     QProgressDialog progress (QObject::tr("Migrating ") + QString::number(numFiles) + QObject::tr(" files")+"\n"+
-    						  QObject::tr("from ") + QDir(datadir).dirName() + "\n"+QObject::tr("to ") + 
-    						  QDir(destDir).dirName(), QString(), 0, numFiles, 0, Qt::WindowSystemMenuHint | Qt::WindowTitleHint);
+    QObject::tr("from ") + QDir(datadir).dirName() + "\n"+QObject::tr("to ") +
+    QDir(destDir).dirName(), QString(), 0, numFiles, 0, Qt::WindowSystemMenuHint | Qt::WindowTitleHint);
     progress.setValue(0);
     progress.setMinimumWidth(300);
     progress.show();
@@ -262,6 +274,13 @@ int main(int argc, char* argv[])
 
 #else
 
+#ifndef Q_OS_LINUX
+// Due to a bug in Qt, creating multiple QApplication instances in a process
+// causes subsequent native file dialog boxes to hang on Fedora 35.
+// See https://bugreports.qt.io/browse/QTBUG-90616
+//
+// Since Linux users can simply use the --legacy command-line argument,
+// we can remove the shift-key check that requires those multiple instances.
 bool shiftKeyPressedAtLaunch(int argc, char *argv[])
 {
     // Reliably detecting the shift key requires a QGuiApplication instance, but
@@ -272,22 +291,69 @@ bool shiftKeyPressedAtLaunch(int argc, char *argv[])
     delete app;
     return keymodifier == Qt::ShiftModifier;
 }
+#endif
+
+
+void optionExit(int exitCode, QString error) {
+    if (exitCode) {
+        error.prepend("Command option error: ");
+    }
+    qCritical() << error << ( R"(
+    Help Menu
+    Option                   Description
+    -p                       Pauses execution for 1 second
+    --profile  <name>        Name of profile. if name does not exist then uses profile tab.
+    --l or --language        Force language prompt
+    --datadir  <folderName>  Use folderName as Oscar Data folder. For relatve paths: <Documents folder>/<relative path>.
+                             If folder does not exist then prompts user.
+    --help                   Displays this menu and exits.
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    --hires                  Enables high Resolution
+    --hiresoff               Disables high Resolution
+#endif
+    -l                       Force login option. Internal OSCAR call from RestartApplication.
+    )" );
+    exit (exitCode);
+}
 
 int main(int argc, char *argv[]) {
-
     QString homeDocs = QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation)+"/";
     QCoreApplication::setApplicationName(getAppName());
     QCoreApplication::setOrganizationName(getDeveloperName());
     QCoreApplication::setOrganizationDomain(getDeveloperDomain());
 
+    #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    HighResolution::init();
+    bool hiResEnabled=false;
+
+    for (int i = 1; i < argc; i++) {
+        if (0 == strcmp(argv[i] ,"--hires"))  {
+            HighResolution::init(HighResolution::HRM_ENABLED);
+        } else if (0 == strcmp(argv[i] ,"--hiresoff"))  {
+            HighResolution::init(HighResolution::HRM_DISABLED);
+        } else if (0 == strcmp(argv[i] ,"--datadir"))  { i++;
+        } else if (0 == strcmp(argv[i] ,"-profile"))  { i++;
+        }
+    }
+    if (HighResolution::isEnabled()) {
+        hiResEnabled=true;
+        QGuiApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
+    }
+    #else
+        // high resolution is enable by default
+    #endif
+
     QSettings settings;
 
     // If shift key was held down when OSCAR was launched, force Software graphics Engine (aka LegacyGFX)
     QString forcedEngine = "";
+#ifndef Q_OS_LINUX
+    // Shift key check is skipped on Linux due to a Qt bug, see comment at shiftKeyPressedAtLaunch().
     if (shiftKeyPressedAtLaunch(argc, argv)){
         settings.setValue(GFXEngineSetting, (unsigned int)GFX_Software);
         forcedEngine = "Software Engine forced by shift key at launch";
     }
+#endif
     // This argument needs to be processed before creating the QApplication,
     // based on sample code at https://doc.qt.io/qt-5/qapplication.html#details
     for (int i = 1; i < argc; ++i) {
@@ -321,8 +387,8 @@ int main(int argc, char *argv[]) {
         QCoreApplication::setAttribute(Qt::AA_UseSoftwareOpenGL);
     }
 
-    QApplication a(argc, argv);
-    QStringList args = a.arguments();
+    QApplication mainapp(argc, argv);
+    QStringList args = mainapp.arguments();
 
 #ifdef Q_OS_WIN
     // QMessageBox must come after the application is created. The graphics engine has to be selected before.
@@ -334,52 +400,6 @@ int main(int argc, char *argv[]) {
     }
 #endif
 
-    QString lastlanguage = settings.value(LangSetting, "").toString();
-    if (lastlanguage.compare("is", Qt::CaseInsensitive))    // Convert code for Hebrew from 'is' to 'he'
-        lastlanguage = "he";
-
-    bool dont_load_profile = false;
-    bool force_data_dir = false;
-    bool changing_language = false;
-    QString load_profile = "";
-
-    if (lastlanguage.isEmpty())
-        changing_language = true;
-
-    for (int i = 1; i < args.size(); i++) {
-        if (args[i] == "-l")
-            dont_load_profile = true;
-//        else if (args[i] == "-d")
-//            force_data_dir = true;
-        else if (args[i] == "--language") {
-            changing_language = true; // reset to force language dialog
-            settings.setValue(LangSetting,"");
-        }
-        // "--legacy" is handle above, as it needs to be processed before creating QApplication.
-        else if (args[i] == "-p")
-            QThread::msleep(1000);
-        else if (args[i] == "--profile") {
-            if ((i+1) < args.size())
-                load_profile = args[++i];
-            else {
-                fprintf(stderr, "Missing argument to --profile\n");
-                exit(1);
-            }
-        } else if (args[i] == "--datadir") { // mltam's idea
-            QString datadir ;
-            if ((i+1) < args.size()) {
-                datadir = args[++i];
-                if (datadir.length() < 2 || datadir.at(1) != QLatin1Char(':'))  // Allow a Windows drive letter (but not UNC)
-                    datadir = homeDocs+datadir;
-                settings.setValue("Settings/AppData", datadir);
-//            force_data_dir = true;
-            } else {
-                fprintf(stderr, "Missing argument to --datadir\n");
-                exit(1);
-            }
-        }
-    }   // end of for args loop
-
     initializeLogger();
     // After initializing the logger, any qDebug() messages will be queued but not written to console
     // until MainWindow is constructed below. In spite of that, we initialize the logger here so that
@@ -388,10 +408,58 @@ int main(int argc, char *argv[]) {
     // The only time this is really noticeable is when initTranslations() presents its language
     // selection QDialog, which waits indefinitely for user input before MainWindow is constructed.
 
+    bool force_data_dir = false;
+    QString load_profile; // null profile means no --profile param
+    for (int i = 1; i < args.size(); i++) {
+        if ((args[i] == "--language") || (args[i] == "--l") ) {
+            settings.setValue(LangSetting,"");
+        } else if ( (args[i] == "-l") || (args[i] == "-nop") || (args[i] == "") ){
+            // do nothing. internal calls that current don't have any functions in main.
+        } else if (args[i] == "-p") {
+            QThread::msleep(1000);
+        } else if (args[i] == "--profile") {
+            if ((i+1) < args.size())
+                load_profile = args[++i];
+            else {
+                // Just view all profiles.
+                load_profile = " ";
+            }
+        } else if (args[i] == "--datadir") { // mltam's idea
+            QString datadir, datadirwas ;
+            if ((i+1) < args.size()) {
+                datadirwas = datadir = args[++i];
+                bool havefullpath = false;
+                if (datadir.length() >= 2) {
+                    havefullpath =    (datadir.at(1) == QLatin1Char(':')) // Allow a Windows drive letter
+                                   || (datadir.at(0) == '/')              // or Linux full path
+                                   || (datadir.at(0) == '\\');
+                }
+                if (!havefullpath) {
+                    datadir = homeDocs+datadir;
+                    qDebug() << "--datadir was:" << datadirwas << "; --datadir is:" << datadir;
+                }
+                settings.setValue("Settings/AppData", datadir);
+//            force_data_dir = true;
+            } else {
+                optionExit(2,"Missing argument to --datadir\n");
+            }
+        } else if (0 == strcmp(argv[i] ,"--hires"))  {      // already handle in 1st scan
+        } else if (0 == strcmp(argv[i] ,"--hiresoff"))  {       // already handle in 1st scan
+        } else if (QString(args[i]).contains("help",Qt::CaseInsensitive)) {
+            optionExit(0,QString(""));
+        } else {
+            optionExit(3,QString("Invalid Argument: '%1'").arg(args[i]));
+        }
+
+    }   // end of for args loop
+
     qDebug().noquote() << "OSCAR starting" << QDateTime::currentDateTime().toString();
     qDebug() << "APP-NAME:" << QCoreApplication::applicationName();
     qDebug() << "APP-PATH:" << QCoreApplication::applicationDirPath();
     qDebug() << "APP-RESOURCES:" << appResourcePath();
+    #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    HighResolution::display(hiResEnabled);
+    #endif
 
 #ifdef QT_DEBUG
     QString relinfo = " debug";
@@ -422,8 +490,9 @@ int main(int argc, char *argv[]) {
 // Moved buildInfo calls to after translation is available as makeBuildInfo includes tr() calls
 
     QStringList info = makeBuildInfo(forcedEngine);
-    for (int i = 0; i < info.size(); ++i)
+    for (int i = 0; i < info.size(); ++i) {
         qDebug().noquote() << info.at(i);
+    }
 
     ////////////////////////////////////////////////////////////////////////////////////////////
     // OpenGL Detection
@@ -540,10 +609,11 @@ int main(int argc, char *argv[]) {
 
     QDir newDir(GetAppData());
 #if QT_VERSION < QT_VERSION_CHECK(5,9,0)
-    if ( ! newDir.exists() || newDir.count() == 0 ) {     // directory doesn't exist yet or is empty, try to migrate old data
+    if ( ! newDir.exists() || newDir.count() == 0 )      // directory doesn't exist yet or is empty, try to migrate old data
 #else
-    if ( ! newDir.exists() || newDir.isEmpty() ) {        // directory doesn't exist yet or is empty, try to migrate old data
+    if ( ! newDir.exists() || newDir.isEmpty() )         // directory doesn't exist yet or is empty, try to migrate old data
 #endif
+    {
         if (QMessageBox::question(nullptr, QObject::tr("Migrate SleepyHead or OSCAR Data?"),
                                   QObject::tr("On the next screen OSCAR will ask you to select a folder with SleepyHead or OSCAR data") +"\n" +
                                   QObject::tr("Click [OK] to go to the next screen or [No] if you do not wish to use any SleepyHead or OSCAR data."),
@@ -581,6 +651,7 @@ int main(int argc, char *argv[]) {
                              QObject::tr("Unable to write to debug log. You can still use the debug pane (Help/Troubleshooting/Show Debug Pane) but the debug log will not be written to disk."));
     }
 
+
     ///////////////////////////////////////////////////////////////////////////////////////////
     // Initialize preferences system (Don't use p_pref before this point!)
     ///////////////////////////////////////////////////////////////////////////////////////////
@@ -590,6 +661,7 @@ int main(int argc, char *argv[]) {
 
     QString language = settings.value(LangSetting, "").toString();
     AppSetting->setLanguage(language);
+    if (!load_profile.isEmpty()) AppSetting->setProfileName(load_profile);
 
     // Set fonts from preferences file
     qDebug() << "App font before Prefs setting" << QApplication::font();
@@ -666,13 +738,18 @@ int main(int argc, char *argv[]) {
     PRS1Loader::Register();
     ResmedLoader::Register();
     IntellipapLoader::Register();
-    FPIconLoader::Register();
     SleepStyleLoader::Register();
+    FPIconLoader::Register();
     WeinmannLoader::Register();
     CMS50Loader::Register();
     CMS50F37Loader::Register();
     MD300W1Loader::Register();
     ViatomLoader::Register();
+    PrismaLoader::Register();
+    ResventLoader::Register();
+    BmcLoader::Register();
+    VREMLoader::Register();
+
 
     // Begin logging device connection activity.
     QString connectionsLogDir = GetLogDir() + "/connections";
@@ -694,9 +771,6 @@ int main(int argc, char *argv[]) {
     // Scan for user profiles
     Profiles::Scan();
 
-    Q_UNUSED(changing_language)
-    Q_UNUSED(dont_load_profile)
-
 #ifndef NO_CHECKUPDATES
     if (check_updates) {
         mainwin->CheckForUpdates(false);
@@ -706,10 +780,10 @@ int main(int argc, char *argv[]) {
     mainwin->SetupGUI();
     mainwin->show();
 
-    int result = a.exec();
-    
+    int result = mainapp.exec();
+
     DeviceConnectionManager::getInstance().record(nullptr);
-    
+
     return result;
 }
 
